/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* Restricted Materials of IBM                                            */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 2006,2007          */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
#ifndef _PRAGMA_COPYRIGHT_
#define _PRAGMA_COPYRIGHT_
#pragma comment(copyright, "@(#) 1.2 src/rsct/lapi/pnsd/nrt.h, lapi, comm_rti, rti0711a 07/04/20 07:58:00\0")
#endif /* _PRAGMA_COPYRIGHT_ */
/*
 * Start Date     : September, 2006
 * Change History :
 *
 *   2006
 *   ----
 *   09/05:  RS F137111 - AIX/GM code drop.
 *   01/19/07: PFC D141243 - Remove nrt_reserve_any_window.
 */
#ifndef _NRT_H
#define _NRT_H

#include <stdint.h>

#define MAX_SPIGOTS               4
#define NRT_MAX_DEVICENAME_SIZE  32
#define NRT_VERSION             420
typedef enum { LEAVE_IN_USE, KILL } clean_option_t;

////////////////////////////////////////////////////////////////////
// Following structures represent values that must be obtained from 
// the resource manager. The *_per_table_input_t structure actually 
// represents values passed in as parameters to the NRT API calls.
//////////////////////////////////////////////////////////////////// 
typedef struct {              
    /////////////////////////////////////////////////////////////////
    // Structure provided by LL as part of Load NRT for each task 
    //     for IB.
    /////////////////////////////////////////////////////////////////
    uint16_t task_id;
    uint16_t win_id;
    uint32_t node_number;                            // provided by HA
    char     device_name[NRT_MAX_DEVICENAME_SIZE]; // e.g, /dev/hpce0

    uint16_t base_lid;
    uint8_t  port_id;
    uint8_t  lmc;
    uint8_t  port_status;  // ??? need this for IB failover/recovery ???
    uint8_t  padding[3];
} nrt_creator_ib_per_task_input_t;

typedef struct {
    /////////////////////////////////////////////////////////////////////////
    // Structure encapsulating info provided by LL as part of Load NRT or
    //     HPCE for each task.
    /////////////////////////////////////////////////////////////////////////
    uint16_t task_id;
    uint16_t win_id;
    uint32_t node_number;                          // provided by RSCT
    char     device_name[NRT_MAX_DEVICENAME_SIZE]; // e.g, /dev/hpce0
} nrt_creator_hpce_per_task_input_t;
   
typedef union {
    nrt_creator_hpce_per_task_input_t hpce_per_task;
    nrt_creator_ib_per_task_input_t   ib_per_task;
} nrt_creator_per_task_input_t;

typedef struct {
    /////////////////////////////////////////////////////////////////////////
    // Structure (or info) provided by LL as part of Load NRT (one per NRT)
    //     for IB and HPCE. Note the prefix nrt_creator* is chosen so as to 
    //     be LL independent.
    /////////////////////////////////////////////////////////////////////////
    uint16_t num_tasks;
    uint16_t job_key;
    uid_t    uid;
    uint64_t net_id;
} nrt_creator_per_table_input_t;

//
// Following structure is used in nrt_adapter_resources() call
//
typedef struct {
    uint32_t node_number;             // to form unique ID
    uint8_t  num_spigots;             // Ports for IB, spigots for HPCE
    uint8_t  padding1[3];             // future use?
    uint16_t lid[MAX_SPIGOTS];        // (IB only) Logical ID of each spigot    
    uint64_t network_id[MAX_SPIGOTS]; // Network ID for each adapter spigot 
    uint8_t  lmc[MAX_SPIGOTS];        // (IB only) Logical mask of each spigot 
    uint8_t  spigot_id[MAX_SPIGOTS];  // Port/Spigot IDs
    uint16_t window_count;            // Count of windows in window_list 
    uint8_t  padding2[6];             // future use?
    uint16_t *window_list;            // array of available windows         
    uint64_t rcontext_block_count;    // available rcxt blocks    
} adap_resources_t;

//
// Structure to request and get info about any (arbitrarily
// selected) reserved window 
//
typedef struct {
    uint32_t node_number;  // to fill in uniq_id structure
    uint8_t  padding[4];  
    char     device_name[NRT_MAX_DEVICENAME_SIZE]; 
                           // to fill in uniq_id structure
       // Note: above needs to be full path (path prefix + device name in NAM)
       //     because of ConfigRM which gives full path to LL 9
    uint64_t net_id;
} any_window_info_t;

//
// Following two structures are for the nrt_status_adapter() call
//
typedef enum { NRT_WIN_UNAVAILABLE, // Initialization stuff in progress
               NRT_WIN_INVALID,     // Not a usable window 
               NRT_WIN_AVAILABLE,   // Ready for NRT load
               NRT_WIN_RESERVED,    // Window reserved, NRT not loaded 
                                    //     (for POE-PMD)
               NRT_WIN_READY,       // NRT loaded
               NRT_WIN_RUNNING      // HAL has gotten NRT from us
} win_state_t;

typedef struct {
        pid_t       client_pid;      // Pid of process that loaded
        uid_t       uid;             // Uid using the window
        uint16_t    window_id;       // Window being reported
        uint16_t    bulk_transfer;   // Is this load using RDMA?
        uint32_t    rcontext_blocks; // rcontexts per window
        win_state_t state;           // Window state
        uint8_t     padding[4];
} nrt_status_t;

typedef enum {
    NRT_SUCCESS,            // 0 - Success
    NRT_EINVAL,             // 1 - Invalid argument
    NRT_EPERM,              // 2 - Caller not authorized
    NRT_PNSDAPI,            // 3 - PNSD API returned an error
    NRT_EADAPTER,           // 4 - Invalid adapter
    NRT_ESYSTEM,            // 5 - System Error occurred
    NRT_EMEM,               // 6 - Memory error
    NRT_EIO,                // 7 - Adapter reports down
    NRT_NO_RDMA_AVAIL,      // 8 - No RDMA windows available 
    NRT_EADAPTYPE,          // 9 - Invalid adapter type
    NRT_BAD_VERSION,        // 10- Version must match nrt_version()
    NRT_EAGAIN,             // 11- Try the call again later
    NRT_WRONG_WINDOW_STATE, // 12- Window in wrong state for this operation
    NRT_UNKNOWN_ADAPTER,    // 13- One (or more) adapters in NRT unknown
    NRT_NO_FREE_WINDOW,     // 14- For reserve any windows (none free)
    NRT_ALREADY_LOADED      // 15- NRT with same key and task 0 config
                            //     already loaded for this device 
} nrt_rc_t;

#ifdef __cplusplus
extern "C" {
#endif // _cplusplus
int nrt_version(void);
int nrt_unload_window(int version, char *adapter_device_string, 
                      uint16_t adapter_type, ushort job_key, 
                      ushort window_id);
int nrt_clean_window(int version, char *adapter_device_string,
                     uint16_t adapter_type,
                     clean_option_t leave_inuse_or_kill, 
                     ushort window_id);
int nrt_rdma_jobs(int version, char *adapter_device_string, 
                  uint16_t adapter_type, uint16_t *job_count,
                  uint16_t **job_keys);
int nrt_load_table_rdma(int version, char *adapter_device_string,
                        uint16_t adapter_type,
                        uint64_t network_id, uid_t uid, pid_t pid, 
                        ushort job_key, char *job_description, 
                        uint use_bulk_transfer, 
                        uint bulk_transfer_resources, int table_size,   
                        nrt_creator_per_task_input_t *per_task_input);
int nrt_adapter_resources(int version, char *adapter_device_string,
                          uint16_t adapter_type,
                          adap_resources_t *adapter_info_OUT);
int nrt_status_adapter(int version, char *adapter_device_string, 
                       uint16_t adapter_type, uint16_t *window_count,
                       nrt_status_t **status_array);
#ifdef __cplusplus
}
#endif // _cplusplus

typedef int (*nrt_version_t)(void);
typedef int (*nrt_unload_window_t)(int version, char *adapter_device_string, 
                        uint16_t adapter_type, ushort job_key, 
                        ushort window_id);
typedef int (*nrt_clean_window_t)(int version, char *adapter_device_string,
                       uint16_t adapter_type,
                       clean_option_t leave_inuse_or_kill, 
                       ushort window_id);
typedef int (*nrt_rdma_jobs_t)(int version, char *adapter_device_string, 
                    uint16_t adapter_type, uint16_t *job_count,
                    uint16_t **job_keys);
typedef int (*nrt_load_table_rdma_t)(int version, char *adapter_device_string,
                          uint16_t adapter_type,
                          uint64_t network_id, uid_t uid, pid_t pid, 
                          ushort job_key, char *job_description, 
                          uint use_bulk_transfer, 
                          uint bulk_transfer_resources, int table_size,   
                          nrt_creator_per_task_input_t *per_task_input);
typedef int (*nrt_adapter_resources_t)(int version, 
                            char *adapter_device_string,
                            uint16_t adapter_type,
                            adap_resources_t *adapter_info_OUT);
typedef int (*nrt_status_adapter_t)(int version, char *adapter_device_string, 
                         uint16_t adapter_type, uint16_t *window_count,
                         nrt_status_t **status_array);
#endif // not _NRT_H
